# Copyright 2023- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class ResCompany(models.Model):
    _inherit = "res.company"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    company_share_line_ids = fields.One2many(
        comodel_name="company.share.line",
        inverse_name="company_id",
    )

    # ------------------------------------------------------
    # Constraints
    # ------------------------------------------------------
    @api.constrains("company_share_line_ids",)
    def _check_share_line(self):
        for company in self:
            company_share_line = company.company_share_line_ids
            share_in = sum(
                company_share_line.filtered(
                    lambda s: s.share_action == "purchase"
                ).mapped("share_number")
            )
            share_out = sum(
                company_share_line.filtered(
                    lambda s: s.share_action == "refund"
                ).mapped("share_number")
            )
            if share_out > share_in:
                raise ValidationError(_("You cannot refund more shares that you have."))

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
